import json
import logging

import pymongo
import redis
import xappy

from config import CHARM_INDEX_DIR, MONGO_HOST, MONGO_PORT

log = logging.getLogger("charm.index")


def setup_indexer(indexer):

    indexer.add_field_action('owner', xappy.FieldActions.INDEX_EXACT)
    indexer.add_field_action('series', xappy.FieldActions.INDEX_EXACT)
    indexer.add_field_action('subordinate', xappy.FieldActions.INDEX_EXACT)
    indexer.add_field_action('provides', xappy.FieldActions.INDEX_EXACT)
    indexer.add_field_action('requires', xappy.FieldActions.INDEX_EXACT)

    # Store content
    indexer.add_field_action('name', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('store_url', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('summary', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('short_url', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('label', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('series', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('owner', xappy.FieldActions.STORE_CONTENT)
    indexer.add_field_action('subordinate', xappy.FieldActions.STORE_CONTENT)

    # Full text search fields
    indexer.add_field_action(
        'name', xappy.FieldActions.INDEX_FREETEXT, weight=10,
        language='en')

    indexer.add_field_action(
        'summary', xappy.FieldActions.INDEX_FREETEXT, weight=5,
        language='en')

    indexer.add_field_action(
        'description', xappy.FieldActions.INDEX_FREETEXT, weight=3,
        language='en')

    indexer.add_field_action(
        'config', xappy.FieldActions.INDEX_FREETEXT, language='en')

    indexer.add_field_action(
        'relations', xappy.FieldActions.INDEX_FREETEXT, language='en')

    indexer.add_field_action(
        'changes', xappy.FieldActions.INDEX_FREETEXT, language='en')


def index_charm(indexer, charm):
    doc = xappy.UnprocessedDocument()

    # Weight critical fields higher for official charms.
    if charm['owner'] == 'charmers':
        weight = 10
    else:
        weight = 1
    doc.fields.append(xappy.Field(
        "name", charm["name"], weight=weight))
    doc.fields.append(xappy.Field(
        "summary", charm["summary"], weight=weight))
    doc.fields.append(xappy.Field(
        "description", charm["description"], weight=weight))

    doc.fields.append(xappy.Field("owner", charm["owner"]))
    doc.fields.append(xappy.Field("short_url", charm["short_url"]))
    doc.fields.append(xappy.Field("label", charm["label"]))
    doc.fields.append(xappy.Field("series", charm["series"]))

    if charm.get('subordinate'):
        doc.fields.append(xappy.Field('subordinate', 'true'))

    if "config" in charm \
       and charm['config'] \
       and 'options' in charm['config'] \
       and charm['config']['options']:
        config_text = []

        for key, option in charm["config"]["options"].items():
            config_text.append(key)
            config_text.append(option.get("description", ""))
        doc.fields.append(xappy.Field("config", " ".join(config_text)))

    relation_text = []

    if "requires" in charm and charm["requires"]:
        for key, option in charm["requires"].items():
            relation_text.append(key)
            relation_text.append(option["interface"])
            doc.fields.append(xappy.Field('requires', option["interface"]))
    if "provides" in charm and charm["provides"]:
        for key, option in charm["provides"].items():
            if not isinstance(option, dict):
                log.warning("invalid charm provides %s", charm['branch_spec'])
                continue
            relation_text.append(key)
            relation_text.append(option["interface"])
            doc.fields.append(xappy.Field("provides", option["interface"]))

    if relation_text:
        doc.fields.append(xappy.Field("relations", " ".join(relation_text)))

    change_text = []
    for change in charm.get("changes", ()):
        change_text.append(change['message'])
        change_text.append(change['committer'])

    if change_text:
        doc.fields.append(xappy.Field("changes", " ".join(change_text)))

    if 'store_url' in charm:
        doc.fields.append(xappy.Field('store_url', charm['store_url']))
    else:
        log.warning("No store url found for %s", charm["_id"])
    doc.id = charm["_id"]
    indexer.replace(doc)
    indexer.flush()


def reindex(db, indexer):
    count = 0
    for charm in db.find():
        log.debug("Indexing %s", charm['branch_spec'])
        try:
            index_charm(indexer, charm)
        except:
            log.error("Indexing charm %s", charm['branch_spec'])
            raise
        count += 1
    log.info("Indexed %d Charms" % count)


def index_queue(db, server, indexer, index_queue="charm-index"):

    count = 0
    while 1:
        count += 1
        item = server.lpop(index_queue)
        if not item:
            return

        try:
            charm_data = json.loads(item)
            charm = db.find_one({"_id": charm_data["branch_spec"]})

            log.debug("Indexing charm %s", charm_data["branch_spec"])
            if charm:
                index_charm(indexer, charm)
            else:
                log.info(
                    "Skipping unknown charm %s", charm_data["branch_spec"])
        except KeyboardInterrupt:
            raise
        except:
            log.exception("Error indexing %s", item)
            continue

    log.info("Indexed %d Charms" % count)


def main():
    logging.basicConfig(
        level=logging.WARNING,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")

    indexer = xappy.IndexerConnection(CHARM_INDEX_DIR)
    setup_indexer(indexer)

    connection = pymongo.Connection(MONGO_HOST, MONGO_PORT)
    db = connection.juju.charms

    # Index queue
    server = redis.Redis()
    index_queue(db, server, indexer)
    #reindex(db, indexer)

    indexer.close()
    log.info("Indexed Repo")

if __name__ == '__main__':
    main()
